@@ -1,29 +1,21 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
-Created on Tue Feb 25 05:45:52 2020
-@author: deborahkhider
-Contains all relevant plotting functions
+Plotting utilities, leveraging Matplotlib.
 """
 
-__all__ = [
-    'set_style',
-    'showfig',
-    'savefig',
-    'closefig',
-]
+__all__ = ['set_style','closefig', 'savefig']
+
 
-# from tkinter import Variable
 import matplotlib.pyplot as plt
 import pathlib
 import matplotlib as mpl
 
 
-
-def plot_scatter_xy(x1, y1,x2,y2, figsize=None, xlabel=None,
+def plot_scatter_xy(x1,y1,x2,y2, figsize=None, xlabel=None,
                     ylabel=None, title=None, xlim=None, ylim=None,
                     savefig_settings=None, ax=None, legend=True, 
-                    plot_kwargs=None, lgd_kwargs=None, mute=False):
+                    plot_kwargs=None, lgd_kwargs=None):
     
     ''' Plot a scatter on top of a line plot.
     
@@ -58,10 +50,6 @@ def plot_scatter_xy(x1, y1,x2,y2, figsize=None, xlabel=None,
         the keyword arguments for ax.legend()
     plot_kwargs : dict
         the keyword arguments for ax.plot()
-    mute : bool
-        if True, the plot will not show;
-         recommend to turn on when more modifications are going to be made on ax
-         (going to be deprecated)
     savefig_settings : dict
         the dictionary of arguments for plt.savefig(); some notes below:
         - "path" must be specified; it can be any existed or non-existed path,
@@ -73,12 +61,11 @@ def plot_scatter_xy(x1, y1,x2,y2, figsize=None, xlabel=None,
     
     ax : the pyplot.axis object
     
-    See Also
+    See also
     -------- 
     
     pyleoclim.utils.plotting.set_style : set different styles for the figures. Should be set before invoking the plotting functions
     pyleoclim.utils.plotting.savefig : save figures
-    pyleoclim.utils.plotting.showfig : equivalent to plt.show(). Platform-dependent
     
     '''
     # handle dict defaults
@@ -115,9 +102,6 @@ def plot_scatter_xy(x1, y1,x2,y2, figsize=None, xlabel=None,
     if 'fig' in locals():
         if 'path' in savefig_settings:
             savefig(fig, settings=savefig_settings)
-        # else:
-        #     if not mute:
-        #         showfig(fig)
         return fig, ax
     else:
         return ax
@@ -125,7 +109,7 @@ def plot_scatter_xy(x1, y1,x2,y2, figsize=None, xlabel=None,
 
 def plot_xy(x, y, figsize=None, xlabel=None, ylabel=None, title=None, 
             xlim=None, ylim=None,savefig_settings=None, ax=None,
-            legend=True, plot_kwargs=None, lgd_kwargs=None, mute=False,
+            legend=True, plot_kwargs=None, lgd_kwargs=None,
             invert_xaxis=False):
     ''' Plot a timeseries
     
@@ -164,7 +148,6 @@ def plot_xy(x, y, figsize=None, xlabel=None, ylabel=None, title=None,
         - "path" must be specified; it can be any existed or non-existed path,
           with or without a suffix; if the suffix is not given in "path", it will follow "format"
         - "format" can be one of {"pdf", "eps", "png", "ps"}
-
     invert_xaxis : bool, optional
         if True, the x-axis of the plot will be inverted
         
@@ -177,9 +160,7 @@ def plot_xy(x, y, figsize=None, xlabel=None, ylabel=None, title=None,
     --------
     
     pyleoclim.utils.plotting.set_style : set different styles for the figures. Should be set before invoking the plotting functions
-    pyleoclim.utils.plotting.savefig : save figures
-    pyleoclim.utils.plotting.showfig : equivalent to plt.show(). Platform-dependent
-        
+    pyleoclim.utils.plotting.savefig : save figures        
     '''
     # handle dict defaults
     savefig_settings = {} if savefig_settings is None else savefig_settings.copy()
@@ -217,194 +198,19 @@ def plot_xy(x, y, figsize=None, xlabel=None, ylabel=None, title=None,
     if 'fig' in locals():
         if 'path' in savefig_settings:
             savefig(fig, settings=savefig_settings)
-        # else:
-        #     if not mute:
-        #         showfig(fig)
         return fig, ax
     else:
         return ax
 
-def stackplot(x, y, figsize=None, xlabel=None, ylabel=None, 
-              xlim=None, ylim=None, title=None,
-              savefig_settings=None, ax=None, style=None, 
-              plot_kwargs=None, mute=False, color=None):
-    ''' Stack plot of timeseries
-    
-    Please not that this function uses a different default style than the Pyleoclim package.
-    To change the style, pass it in the set_style argument    
-      
-    Parameters
-    ----------
-    x : nested list
-        x values of individual timeseries
-    y : nested list
-        y values of individual timeseries
-    figsize : list
-        a list of two integers indicating the figure size
-    xlabel : str
-        label for x-axis
-    ylabel : str
-        label for y-axis
-    xlim : list
-        set the limits of the x axis
-    ylim : nested list
-        set the limits of the y axes. Should be the same length as y
-    title : str
-        the title for the figure
-    ax : pyplot.axis
-        the pyplot.axis object
-    style : str
-        style of the plot. See set_style
-    plot_kwargs : dict
-        the keyword arguments for ax.plot()
-    mute : bool
-        if True, the plot will not show;
-          recommend to turn on when more modifications are going to be made on ax
-         (going to be deprecated)
-    savefig_settings : dict
-        the dictionary of arguments for plt.savefig(); some notes below:
-        - "path" must be specified; it can be any existed or non-existed path,
-          with or without a suffix; if the suffix is not given in "path", it will follow "format"
-        - "format" can be one of {"pdf", "eps", "png", "ps"}
-    color : list
-        list of colors chosen from a particular coloring scheme with the same size as y to distinguish different series
-        
-    See Also
-    --------
-    
-    pyleoclim.utils.plotting.set_style : set different styles for the figures. Should be set before invoking the plotting functions
-    pyleoclim.utils.plotting.savefig : save figures
-    pyleoclim.utils.plotting.showfig : equivalent to plt.show(). Platform-dependent
-    
-      '''
-    savefig_settings = {} if savefig_settings is None else savefig_settings.copy()
-    plot_kwargs = {} if plot_kwargs is None else plot_kwargs.copy()
-       
-    min_x = min([a for i in x for a in i])
-    max_x = max([a for i in x for a in i])
-    if style is None:
-        set_style('journal_spines')
-
-    if ax is None:
-        fig, ax = plt.subplots(len(x),1,figsize=figsize,sharex=True)
-
-    fig.subplots_adjust(hspace=0.0001)
-    for i in range(len(x)):
-        if color is not None:
-
-            ax[i].plot(x[i], y[i], color=color[i])
-        else:
-            ax[i].plot(x[i], y[i])
-            
-        #Set the limits    
-        if xlim is not None:
-            ax[i].set_xlim(xlim)
-        else:
-            ax[i].set_xlim(0, max_x)
-        
-        if ylim is not None:
-            ax[i].set_ylim(ylim[i])
-        
-        # ax[i].set_xticks(np.arange(min_x,max_x))
-        if xlabel is None:
-            ax[i].set_xlabel('Time')
-        if ylabel is None:
-            ax[i].set_ylabel('Series {}'.format(i + 1))
-        if i % 2 == 1:
-            ax[i].yaxis.set_label_position("right")
-            ax[i].yaxis.tick_right()
-
-            ax[i].spines['left'].set_visible(False)
-        else:
-            ax[i].spines['right'].set_visible(False)
-        ax[i].spines['top'].set_visible(False)
-        if i!=len(x)-1:
-            ax[i].spines['bottom'].set_visible(False)
-        if color is not None:
-            ax[i].tick_params(axis='y', colors=color[i])
-            ax[i].yaxis.label.set_color(color[i])
-
-        ax[i].tick_params(axis='both', which='major', labelsize=12)
-
-
-    if 'fig' in locals():
-        if 'path' in savefig_settings:
-            savefig(fig, settings=savefig_settings)
-        # else:
-        #     if not mute:
-        #         showfig(fig)
-        return fig, ax
-    else:
-        return ax
-
-# ----------
-# utilities
-# ----------
-def in_notebook():
-    ''' Check if the code is executed in a Jupyter notebook
-    
-    Returns
-    -------
-    
-    bool
-    
-    '''
-    try:
-        from IPython import get_ipython
-        if 'IPKernelApp' not in get_ipython().config:  # pragma: no cover
-            return False
-    except ImportError:
-        return False
-    return True
-
-
-def showfig(fig, close=False):
-    '''Show the figure
-
-    Parameters
-    ----------
-    fig : matplotlib.pyplot.figure
-        The matplotlib figure object
-
-    close : bool
-        if True, close the figure automatically
-
-    See Also
-    --------
-    pyleoclim.utils.plotting.savefig : saves a figure to a specific path
-    pyleoclim.utils.plotting.in_notebook: Functions to sense a notebook environment
-
-    '''
-    # if in_notebook:
-    #     try:
-    #         from IPython.display import display
-    #     except ImportError as error:
-    #         # Output expected ImportErrors.
-    #         print(f'{error.__class__.__name__}: {error.message}')
-
-    #     display(fig)
-
-    # else:
-    #     plt.show()
-
-    plt.show()
-
-    if close:
-        closefig(fig)
 
 def closefig(fig=None):
-    '''Show the figure
+    '''Close the figure
 
     Parameters
     ----------
     fig : matplotlib.pyplot.figure
         The matplotlib figure object
 
-    See Also
-    --------
-    pyleoclim.utils.plotting.savefig : saves a figure to a specific path
-    pyleoclim.utils.plotting.in_notebook: Functions to sense a notebook environment
-
     '''
     if fig is not None:
         plt.close(fig)
@@ -426,11 +232,9 @@ def savefig(fig, path=None, settings={}, verbose=True):
           it can be any existed or non-existed path, with or without a suffix;
           if the suffix is not given in "path", it will follow "format"
         - "format" can be one of {"pdf", "eps", "png", "ps"}
+    verbose : bool, {True,False}
+        If True, print the path of the saved file.
         
-    See Also
-    --------
-    
-    pyleoclim.utils.plotting.showfig : returns a visual of the figure. 
     '''
     if path is None and 'path' not in settings:
         raise ValueError('"path" must be specified, either with the keyword argument or be specified in `settings`!')
@@ -462,7 +266,7 @@ def set_style(style='journal', font_scale=1.0):
     ''' Modify the visualization style
     
     This function is inspired by [Seaborn](https://github.com/mwaskom/seaborn).
-    See a demo in the example_notebooks folder on GitHub to look at the different styles
+   
     
     Parameters
     ----------
