@@ -1,21 +1,25 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
-Created on Tue Feb 25 09:23:29 2020
+Utilities for spectral analysis, including WWZ, CWT, Lomb-Scargle, MTM, and Welch.
+Designed for NumPy arrays, either evenly spaced or not (method-dependent).
 
-@author: deborahkhider
+All spectral methods must return a dictionary containing one vector for the 
+frequency axis and the power spectral density (PSD).
 
-Sectral analysis functions
+Additional utilities help compute an optimal frequency vector or estimate scaling exponents.
 """
 
 import numpy as np
 from scipy import signal
 import nitime.algorithms as nialg
+import statsmodels.api as sm
 import collections
 import warnings
 
 __all__ = [
     'wwz_psd',
+    'cwt_psd',
     'mtm',
     'lomb_scargle',
     'welch',
@@ -36,7 +40,6 @@ from .wavelet import (
     wwa2psd,
     cwt,
 )
-#from .tsutils import clean_ts, interp, bin
 
 #-----------
 #Wrapper
@@ -51,12 +54,15 @@ def welch(ys, ts, window='hann',nperseg=None, noverlap=None, nfft=None,
            return_onesided=True, detrend = None, sg_kwargs = None,
            gaussianize=False, standardize=False,
            scaling='density', average='mean'):
-    '''Estimate power spectral density using Welch's method
+    '''Estimate power spectral density using Welch's periodogram
 
     Wrapper for the function implemented in scipy.signal.welch
     See https://docs.scipy.org/doc/scipy/reference/generated/scipy.signal.welch.html for details.
 
-    Welch's method is an approach for spectral density estimation. It computes an estimate of the power spectral density by dividing the data into overlapping segments, computing a modified periodogram for each segment and averaging the periodograms.
+    Welch's method is an approach for spectral density estimation. It computes 
+    an estimate of the power spectral density by dividing the data into overlapping
+    segments, computing a modified periodogram for each segment and averaging 
+    the periodograms to lower the estimator's variance.
 
     Parameters
     ----------
@@ -114,7 +120,7 @@ def welch(ys, ts, window='hann',nperseg=None, noverlap=None, nfft=None,
       scaling : {"density,"spectrum}
           Selects between computing the power spectral density (‘density’) where Pxx has units of V**2/Hz and computing the power spectrum (‘spectrum’) where Pxx has units of V**2, if x is measured in V and fs is measured in Hz. Defaults to ‘density'
       average : {'mean','median'}
-          Method to use when averaging periodograms. Defaults to ‘mean’.
+          Method to use when combining periodograms. Defaults to ‘mean’.
 
     Returns
     -------
@@ -126,16 +132,21 @@ def welch(ys, ts, window='hann',nperseg=None, noverlap=None, nfft=None,
 
     See also
     --------
-    pyleoclim.utils.spectral.periodogram : Estimate power spectral density using a periodogram
-    pyleoclim.utils.spectral.mtm : Retuns spectral density using a multi-taper method
-    pyleoclim.utils.spectral.lomb_scargle : Return the computed periodogram using lomb-scargle algorithm
-    pyleoclim.utils.spectral.wwz_psd : Return the psd of a timeseries using wwz method.
+    pyleoclim.utils.spectral.periodogram : Spectral density estimation using a Blackman-Tukey periodogram
+    pyleoclim.utils.spectral.mtm : Spectral density estimation using the multi-taper method
+    pyleoclim.utils.spectral.lomb_scargle : Lomb-scargle priodogram
+    pyleoclim.utils.spectral.wwz_psd : Spectral estimation using the Weighted Wavelet Z-transform
+    pyleoclim.utils.spectral.cwt_psd : Spectral estimation using the Continuous Wavelet Transform
     pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay
-    pyleoclim.utils.tsutils.detrend : Detrending method
+    pyleoclim.utils.tsutils.detrend : detrending functionalities using 4 possible methods  
+    pyleoclim.utils.tsutils.gaussianize_1d: Quantile maps a 1D array to a Gaussian distribution 
+    pyleoclim.utils.tsutils.standardize: Centers and normalizes a given time series.
 
     References
     ----------
-    P. Welch, “The use of the fast Fourier transform for the estimation of power spectra: A method based on time averaging over short, modified periodograms”, IEEE Trans. Audio Electroacoust. vol. 15, pp. 70-73, 1967.
+    P. Welch, “The use of the fast Fourier transform for the estimation of power spectra: 
+        A method based on time averaging over short, modified periodograms”, 
+        IEEE Trans. Audio Electroacoust. vol. 15, pp. 70-73, 1967.
 
     '''
 
@@ -184,10 +195,10 @@ def welch(ys, ts, window='hann',nperseg=None, noverlap=None, nfft=None,
 def mtm(ys, ts, NW=None, BW=None, detrend = None, sg_kwargs=None,
            gaussianize=False, standardize=False, adaptive=False, jackknife=True,
            low_bias=True, sides='default', nfft=None):
-    ''' Retuns spectral density using a multi-taper method.
+    ''' Spectral density using the multi-taper method.
 
 
-    Based on the function in the time series analysis for neuroscience toolbox: http://nipy.org/nitime/api/generated/nitime.algorithms.spectral.html
+    Based on the nitime package: http://nipy.org/nitime/api/generated/nitime.algorithms.spectral.html
 
     Parameters
     ----------
@@ -241,12 +252,15 @@ def mtm(ys, ts, NW=None, BW=None, detrend = None, sg_kwargs=None,
 
     See Also
     --------
-    pyleoclim.utils.spectral.periodogram : Estimate power spectral density using a periodogram
-    pyleoclim.utils.spectral.welch : Retuns spectral density using the welch method
-    pyleoclim.utils.spectral.lomb_scargle : Return the computed periodogram using lomb-scargle algorithm
-    pyleoclim.utils.spectral.wwz_psd : Return the psd of a timeseries using wwz method.
+    pyleoclim.utils.spectral.periodogram : Spectral density estimation using a Blackman-Tukey periodogram
+    pyleoclim.utils.spectral.welch : spectral estimation using Welch's periodogram
+    pyleoclim.utils.spectral.lomb_scargle : Lomb-scargle priodogram
+    pyleoclim.utils.spectral.wwz_psd : Spectral estimation using the Weighted Wavelet Z-transform
+    pyleoclim.utils.spectral.cwt_psd : Spectral estimation using the Continuous Wavelet Transform
     pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay
-    pyleoclim.utils.tsutils.detrend : Detrending method
+    pyleoclim.utils.tsutils.detrend : detrending functionalities using 4 possible methods  
+    pyleoclim.utils.tsutils.gaussianize_1d: Quantile maps a 1D array to a Gaussian distribution 
+    pyleoclim.utils.tsutils.standardize: Centers and normalizes a given time series.
 
     '''
     # preprocessing
@@ -295,9 +309,10 @@ def lomb_scargle(ys, ts, freq=None, freq_method='lomb_scargle',
                  gaussianize=False,
                  standardize=False,
                  average='mean'):
-    """ Return the computed periodogram using lomb-scargle algorithm
+    """ Lomb-scargle priodogram
 
-    Uses the lombscargle implementation from scipy.signal: https://scipy.github.io/devdocs/generated/scipy.signal.lombscargle.html#scipy.signal.lombscargle
+    Appropriate for unevenly-spaced arrays.
+    Uses the lomb-scargle implementation from scipy.signal: https://scipy.github.io/devdocs/generated/scipy.signal.lombscargle.html#scipy.signal.lombscargle
 
     Parameters
     ----------
@@ -373,12 +388,16 @@ def lomb_scargle(ys, ts, freq=None, freq_method='lomb_scargle',
 
     See Also
     --------
+    
     pyleoclim.utils.spectral.periodogram : Estimate power spectral density using a periodogram
     pyleoclim.utils.spectral.mtm : Retuns spectral density using a multi-taper method
     pyleoclim.utils.spectral.welch : Returns power spectral density using the Welch method
-    pyleoclim.utils.spectral.wwz_psd : Return the psd of a timeseries using wwz method.
+    pyleoclim.utils.spectral.wwz_psd : Spectral estimation using the Weighted Wavelet Z-transform
+    pyleoclim.utils.spectral.cwt_psd : Spectral estimation using the Continuous Wavelet Transform
     pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay
-    pyleoclim.utils.tsutils.detrend : Detrending method
+    pyleoclim.utils.tsutils.detrend : detrending functionalities using 4 possible methods  
+    pyleoclim.utils.tsutils.gaussianize_1d: Quantile maps a 1D array to a Gaussian distribution 
+    pyleoclim.utils.tsutils.standardize: Centers and normalizes a given time series.
 
     References
     ----------
@@ -500,7 +519,7 @@ def periodogram(ys, ts, window='hann', nfft=None,
            return_onesided=True, detrend = None, sg_kwargs=None,
            gaussianize=False, standardize=False,
            scaling='density'):
-    ''' Estimate power spectral density using a periodogram
+    ''' Spectral density estimation using a Blackman-Tukey periodogram
 
     Based on the function from scipy: https://docs.scipy.org/doc/scipy/reference/generated/scipy.signal.periodogram.html
 
@@ -569,9 +588,12 @@ def periodogram(ys, ts, window='hann', nfft=None,
     pyleoclim.utils.spectral.welch : Estimate power spectral density using the welch method
     pyleoclim.utils.spectral.mtm : Retuns spectral density using a multi-taper method
     pyleoclim.utils.spectral.lomb_scargle : Return the computed periodogram using lomb-scargle algorithm
-    pyleoclim.utils.spectral.wwz_psd : Return the psd of a timeseries using wwz method.
+    pyleoclim.utils.spectral.wwz_psd : Spectral estimation using the Weighted Wavelet Z-transform
+    pyleoclim.utils.spectral.cwt_psd : Spectral estimation using the Continuous Wavelet Transform
     pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay
-    pyleoclim.utils.tsutils.detrend : Detrending method
+    pyleoclim.utils.tsutils.detrend : detrending functionalities using 4 possible methods  
+    pyleoclim.utils.tsutils.gaussianize_1d: Quantile maps a 1D array to a Gaussian distribution 
+    pyleoclim.utils.tsutils.standardize: Centers and normalizes a given time series.
 
     '''
     ts = np.array(ts)
@@ -615,19 +637,19 @@ def periodogram(ys, ts, window='hann', nfft=None,
 def wwz_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,
             tau=None, c=1e-3, nproc=8,
             detrend=False, sg_kwargs=None, gaussianize=False,
-            standardize=False, Neff=3, anti_alias=False, avgs=2,
+            standardize=False, Neff_threshold=3, anti_alias=False, avgs=2,
             method='Kirchner_numba', wwa=None, wwz_Neffs=None, wwz_freq=None):
-    ''' Returns the power spectral density (PSD) of a timeseries using the Weighted Wavelet Z-transform
-
+    ''' Spectral estimation using the Weighted Wavelet Z-transform
+    
     The Weighted wavelet Z-transform (WWZ) is based on Morlet wavelet spectral estimation, using
-    least squares minimization to suppress the energy leakage caused by the data gaps.
+    least squares minimization to suppress the energy leakage caused by data gaps.
     WWZ does not rely on interpolation or detrending, and is appropriate for unevenly-spaced datasets.
     In particular, we use the variant of Kirchner & Neal (2013), in which basis rotations mitigate the
     numerical instability that occurs in pathological cases with the original algorithm (Foster, 1996).
     The WWZ method has one adjustable parameter, a decay constant `c` that balances the time and frequency
     resolutions of the analysis. The smaller this constant is, the sharper the peaks.
-     We choose the value 1e-3 to obtain smooth spectra that lend themselves to better scaling exponent estimation,
-     while still capturing the main periodicities.
+    The default value is 1e-3 to obtain smooth spectra that lend themselves to better scaling exponent
+    estimation, while still capturing the main periodicities. 
 
      Note that scalogram applications use the larger value (8π2)−1, justified elsewhere (Foster, 1996).
 
@@ -683,8 +705,8 @@ def wwz_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,
         - 'Kirchner_f2py':  the method Kirchner adapted from Foster, implemented with f2py for acceleration;
         - 'Kirchner_numba':  the method Kirchner adapted from Foster, implemented with Numba for acceleration (default);
 
-    Neff : int
-        effective number of points
+    Neff_threshold : int
+        threshold for the effective number of points
     anti_alias : bool
         If True, uses anti-aliasing
     avgs : int
@@ -718,8 +740,11 @@ def wwz_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,
     pyleoclim.utils.spectral.mtm : Retuns spectral density using a multi-taper method
     pyleoclim.utils.spectral.lomb_scargle : Return the computed periodogram using lomb-scargle algorithm
     pyleoclim.utils.spectral.welch : Estimate power spectral density using the Welch method
+    pyleoclim.utils.spectral.cwt_psd : Spectral estimation using the Continuous Wavelet Transform
     pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay
-    pyleoclim.utils.tsutils.detrend : Detrending method
+    pyleoclim.utils.tsutils.detrend : detrending functionalities using 4 possible methods  
+    pyleoclim.utils.tsutils.gaussianize_1d: Quantile maps a 1D array to a Gaussian distribution 
+    pyleoclim.utils.tsutils.standardize: Centers and normalizes a given time series.
 
     References
     ----------
@@ -734,14 +759,14 @@ def wwz_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,
     # get wwa but AR1_q is not needed here so set nMC=0
     #  wwa, _, _, coi, freq, _, Neffs, _ = wwz(ys_cut, ts_cut, freq=freq, tau=tau, c=c, nproc=nproc, nMC=0,
     if wwa is None or wwz_Neffs is None or wwz_freq is None:
-        res_wwz = wwz(ys_cut, ts_cut, freq=freq, tau=tau, c=c, nproc=nproc, nMC=0,
+        res_wwz = wwz(ys_cut, ts_cut, freq=freq, tau=tau, c=c, nproc=nproc,
                   detrend=detrend, sg_kwargs=sg_kwargs,
                   gaussianize=gaussianize, standardize=standardize, method=method)
         wwa = res_wwz.amplitude
         wwz_Neffs = res_wwz.Neffs
         wwz_freq = res_wwz.freq
-
-    psd = wwa2psd(wwa, ts_cut, wwz_Neffs, freq=wwz_freq, Neff=Neff, anti_alias=anti_alias, avgs=avgs)
+        
+    psd = wwa2psd(wwa, ts_cut, wwz_Neffs, freq=wwz_freq, Neff_threshold=Neff_threshold, anti_alias=anti_alias, avgs=avgs)
     Results = collections.namedtuple('Results', ['psd', 'freq'])
     res = Results(psd=psd, freq=freq)
 
@@ -750,8 +775,8 @@ def wwz_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,
 def cwt_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,scale = None, 
             detrend=False,sg_kwargs={}, gaussianize=False, standardize =False, pad=False, 
             mother='MORLET',param=None, cwt_res=None):
-    '''
-    Wrapper function to implement Torrence and Compo continuous wavelet transform
+    ''' Spectral estimation using the continuous wavelet transform
+    Uses the Torrence and Compo [1998] continuous wavelet transform implementation
 
     Parameters
     ----------
@@ -803,11 +828,16 @@ def cwt_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,scale = None,
     See also
     --------
     
-    pyleoclim.utils.wavelet.make_freq_vector : make the frequency vector with various methods
-    
+    pyleoclim.utils.wavelet.make_freq_vector : make the frequency vector with various methods    
     pyleoclim.utils.wavelet.cwt: Torrence and Compo implementation of the continuous wavelet transform 
-    
-    pyleoclim.utils.tsutils.detrend : detrending functionalities in Pyleoclim
+    pyleoclim.utils.spectral.periodogram : Spectral estimation using Blackman-Tukey's periodogram
+    pyleoclim.utils.spectral.mtm : Spectral estimation using the multi-taper method
+    pyleoclim.utils.spectral.lomb_scargle : Spectral estimation using the lomb-scargle periodogram
+    pyleoclim.utils.spectral.welch : Spectral estimation using Welch's periodogram
+    pyleoclim.utils.spectral.wwz_psd : Spectral estimation using the Weighted Wavelet Z-transform
+    pyleoclim.utils.tsutils.detrend : detrending functionalities using 4 possible methods  
+    pyleoclim.utils.tsutils.gaussianize_1d: Quantile maps a 1D array to a Gaussian distribution 
+    pyleoclim.utils.tsutils.standardize: Centers and normalizes a given time series.
     
     References
     ----------
@@ -833,4 +863,233 @@ def cwt_psd(ys, ts, freq=None, freq_method='log', freq_kwargs=None,scale = None,
     Results = collections.namedtuple('Results', ['psd', 'freq','scale','mother','param'])
     res = Results(psd=psd, freq=cwt_res.freq, scale=cwt_res.scale, mother=cwt_res.mother,param=cwt_res.param)
 
-    return res
\ No newline at end of file
+    return res
+
+def beta_estimation(psd, freq, fmin=None, fmax=None, logf_binning_step='max', verbose=False):
+    ''' Estimate the scaling exponent of a power spectral density.
+    
+    Models the spectrum as :math: `S(f) \propto 1/f^{\beta}`. For instance:
+    - :math: `\beta = 0` corresponds to white noise
+    - :math: `\beta = 1` corresponds to pink noise
+    - :math: `\beta = 2` corresponds to red noise (Brownian motion)
+
+    
+    Parameters
+    ----------
+
+    psd : array
+        the power spectral density
+    freq : array
+        the frequency vector
+    fmin : float
+        the min of frequency range for beta estimation
+    fmax : float
+        the max of frequency range for beta estimation
+    verbose : bool
+         if True, will print out debug information
+
+    Returns
+    -------
+
+    beta : float
+        the estimated slope
+    f_binned : array
+        binned frequency vector
+    psd_binned : array
+        binned power spectral density
+    Y_reg : array
+        prediction based on linear regression
+
+    '''
+    # drop the PSD at frequency zero
+    if freq[0] == 0:
+        psd = psd[1:]
+        freq = freq[1:]
+
+    if fmin is None or fmin == 0:
+        fmin = np.min(freq)
+
+    if fmax is None:
+        fmax = np.max(freq)
+
+    Results = collections.namedtuple('Results', ['beta', 'f_binned', 'psd_binned', 'Y_reg', 'std_err'])
+    if np.max(freq) < fmax or np.min(freq) > fmin:
+        if verbose:
+            print(fmin, fmax)
+            print(np.min(freq), np.max(freq))
+            print('WRONG')
+        res = Results(beta=np.nan, f_binned=np.nan, psd_binned=np.nan, Y_reg=np.nan, std_err=np.nan)
+        return res
+
+    # frequency binning start
+    fminindx = np.where(freq >= fmin)[0][0]
+    fmaxindx = np.where(freq <= fmax)[0][-1]
+
+    if fminindx >= fmaxindx:
+        res = Results(beta=np.nan, f_binned=np.nan, psd_binned=np.nan, Y_reg=np.nan, std_err=np.nan)
+        return res
+
+    logf = np.log(freq)
+    if logf_binning_step == 'max':
+        logf_step = np.max(np.diff(logf))
+    elif logf_binning_step == 'first':
+        logf_step = logf[fminindx+1] - logf[fminindx]
+    else:
+        raise ValueError('the option for logf_binning_step is unknown')
+
+    logf_start = logf[fminindx]
+    logf_end = logf[fmaxindx]
+    logf_binedges = np.arange(logf_start, logf_end+logf_step, logf_step)
+
+    n_intervals = np.size(logf_binedges)-1
+    logpsd_binned = np.empty(n_intervals)
+    logf_binned = np.empty(n_intervals)
+
+    logpsd = np.log(psd)
+
+    for i in range(n_intervals):
+        lb = logf_binedges[i]
+        ub = logf_binedges[i+1]
+        q = np.where((logf > lb) & (logf <= ub))
+
+        logpsd_binned[i] = np.nanmean(logpsd[q])
+        logf_binned[i] = (ub + lb) / 2
+
+    f_binned = np.exp(logf_binned)
+    psd_binned = np.exp(logpsd_binned)
+    # frequency binning end
+
+    # linear regression below
+    Y = np.log10(psd_binned)
+    X = np.log10(f_binned)
+    X_ex = sm.add_constant(X)
+
+    # note below: 'drop' is used for missing, so NaNs will be removed, and we need to put it back in the end
+    model = sm.OLS(Y, X_ex, missing='drop')
+    results = model.fit()
+
+    if np.size(results.params) < 2:
+        beta = np.nan
+        Y_reg = np.nan
+        std_err = np.nan
+    else:
+        beta = -results.params[1]  # the slope we want
+        Y_reg_raw = 10**model.predict(results.params)  # prediction based on linear regression
+        # handeling potential NaNs in psd_binned
+        Y_reg = []
+        i = 0
+        for psd in psd_binned:
+            if np.isnan(psd):
+                Y_reg.append(np.nan)
+            else:
+                Y_reg.append(Y_reg_raw[i])
+                i += 1
+
+        Y_reg = np.array(Y_reg)
+
+        std_err = results.bse[1]
+
+    res = Results(beta=beta, f_binned=f_binned, psd_binned=psd_binned, Y_reg=Y_reg, std_err=std_err)
+
+    return res
+
+def beta2Hurst(beta):
+    ''' Translates spectral slope to Hurst exponent
+
+    Parameters
+    ----------
+
+    beta : float
+        the estimated slope of a power spectral density :math: `S(f) \propto 1/f^{\beta}`
+        
+    Returns
+    -------
+
+    H : float
+        Hurst index, should be in (0, 1)
+
+    References
+    ----------
+
+    Equation 2 in http://www.bearcave.com/misl/misl_tech/wavelets/hurst/
+    
+    See also
+    --------
+    pyleoclim.utils.spectral.beta_estimation: Estimate the scaling exponent of a power spectral density.
+    
+    '''
+    H = (beta-1)/2
+
+    return H
+
+def psd_ar(var_noise, freq, ar_params, f_sampling):
+    ''' Theoretical power spectral density (PSD) of an autoregressive model
+
+    Parameters
+    ----------
+
+    var_noise : float
+        the variance of the noise of the AR process
+    freq : array
+        vector of frequency
+    ar_params : array
+        autoregressive coefficients, not including zero-lag
+    f_sampling : float
+        sampling frequency
+
+    Returns
+    -------
+
+    psd : array
+        power spectral density
+
+    '''
+    p = np.size(ar_params)
+
+    tmp = np.ndarray(shape=(p, np.size(freq)), dtype=complex)
+    for k in range(p):
+        tmp[k, :] = np.exp(-1j*2*np.pi*(k+1)*freq/f_sampling)
+
+    psd = var_noise / np.absolute(1-np.sum(ar_params*tmp, axis=0))**2
+
+    return psd
+
+
+
+def psd_fBM(freq, ts, H):
+    ''' Theoretical power spectral density of a fractional Brownian motion
+
+    Parameters
+    ----------
+
+    freq : array
+        vector of frequency
+    ts : array
+        the time axis of the time series
+    H : float
+        Hurst exponent, should be in (0, 1)
+
+    Returns
+    --------
+
+    psd : array
+        power spectral density
+
+    References
+    ----------
+
+    Flandrin, P. On the spectrum of fractional Brownian motions.
+    IEEE Transactions on Information Theory 35, 197–199 (1989).
+
+    '''
+    nf = np.size(freq)
+    psd = np.ndarray(shape=(nf))
+    T = np.max(ts) - np.min(ts)
+
+    omega = 2 * np.pi * freq
+
+    for k in range(nf):
+        tmp = 2 * omega[k] * T
+        psd[k] = (1 - 2**(1 - 2*H)*np.sin(tmp)/tmp) / np.abs(omega[k])**(1 + 2*H)
+
+    return psd
