@@ -1,11 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
-Created on Tue Feb 25 06:43:14 2020
-
-@author: deborahkhider, fzhu, jeg
-
-Utilities to manipulate timeseries
+Utilities to manipulate timeseries - useful for preprocessing prior to analysis
 """
 
 __all__ = [
@@ -13,16 +9,12 @@ __all__ = [
     'bin',
     'interp',
     'gkernel',
-    'increments',
     'standardize',
     'ts2segments',
     'annualize',
     'gaussianize',
-    'gaussianize_single',
     'detrend',
-    'detect_outliers',
-    'remove_outliers',
-    'eff_sample_size'
+    'remove_outliers'
 ]
 
 
@@ -43,7 +35,7 @@ import statsmodels.tsa.stattools as sms
 
 import math
 from sys import exit
-from .plotting import plot_scatter_xy,plot_xy,savefig,showfig
+from .plotting import plot_scatter_xy, plot_xy, savefig
 from .filter import savitzky_golay
 
 from .tsbase import (
@@ -53,8 +45,7 @@ from .tsbase import (
 def simple_stats(y, axis=None):
     """ Computes simple statistics
 
-    Computes the mean, median, min, max, standard deviation, and interquartile
-    range of a numpy array y, ignoring NaNs.
+    Computes the mean, median, min, max, standard deviation, and interquartile range of a numpy array y, ignoring NaNs.
 
     Parameters
     ----------
@@ -62,7 +53,7 @@ def simple_stats(y, axis=None):
     y: array
         A Numpy array
     axis : int, tuple of ints
-        Optional. Axis or Axes along which the means
+        Axis or Axes along which the means
         are computed, the default is to compute the mean of the flattened
         array. If a tuple of ints, performed over multiple axes
 
@@ -81,6 +72,7 @@ def simple_stats(y, axis=None):
         standard deviation of y, ignoring NaNs
     IQR : float
         Interquartile range of y along specified axis, ignoring NaNs
+        
     """
     # make sure that y is an array
     y = np.array(y, dtype='float64')
@@ -123,9 +115,16 @@ def bin(x, y, bin_size=None, start=None, stop=None, evenly_spaced = True):
     bins : array
         The bins (centered on the median, i.e., the 100-200 bin is 150)
     n : array
-        number of data points in each bin
+        Number of data points in each bin
     error : array
-        the standard error on the mean in each bin
+        The standard error on the mean in each bin
+
+    See also
+    --------
+
+    pyleoclim.utils.tsutils.gkernel : Coarsen time resolution using a Gaussian kernel
+
+    pyleoclim.utils.tsutils.interp : Interpolate y onto a new x-axis
 
     """
 
@@ -178,8 +177,7 @@ def bin(x, y, bin_size=None, start=None, stop=None, evenly_spaced = True):
 
 
 def gkernel(t,y, h = 3.0, step=None,start=None,stop=None, step_style = 'max'):
-    '''
-    Coarsen time resolution using a Gaussian kernel
+    '''Coarsen time resolution using a Gaussian kernel
 
     Parameters
     ----------
@@ -188,20 +186,21 @@ def gkernel(t,y, h = 3.0, step=None,start=None,stop=None, step_style = 'max'):
     
     y  : 1d array
         values on the original time axis
+        
+    h  : float 
+        kernel e-folding scale
     
     step : float
         The interpolation step. Default is max spacing between consecutive points.
-   
-    step_style : 'string'
-            step style to be applied from `increments` [default = 'max']
-    
-    start : float
+
+        start : float
         where/when to start the interpolation. Default is min(t).
         
     stop : float
         where/when to stop the interpolation. Default is max(t).
-    
-    h  : scalar;  kernel e-folding scale
+   
+    step_style : str
+            step style to be applied from 'increments' [default = 'max']
 
     Returns
     -------
@@ -217,6 +216,16 @@ def gkernel(t,y, h = 3.0, step=None,start=None,stop=None, step_style = 'max'):
     Rehfeld, K., Marwan, N., Heitzig, J., and Kurths, J.: Comparison of correlation analysis
     techniques for irregularly sampled time series, Nonlin. Processes Geophys.,
     18, 389–404, https://doi.org/10.5194/npg-18-389-2011, 2011.
+
+    See also
+    --------
+
+    pyleoclim.utils.tsutils.increments : Establishes the increments of a numerical array
+
+    pyleoclim.utils.tsutils.bin : Bin the values
+
+    pyleoclim.utils.tsutils.interp : Interpolate y onto a new x-axis
+
     '''
 
     if len(t) != len(y):
@@ -232,7 +241,7 @@ def gkernel(t,y, h = 3.0, step=None,start=None,stop=None, step_style = 'max'):
         stop = np.nanmax(np.asarray(t))
     
     # Get the uniform time axis.
-    tc = np.arange(start,stop,step)
+    tc = np.arange(start,stop+step,step)
         
 
     kernel = lambda x, s : 1.0/(s*np.sqrt(2*np.pi))*np.exp(-0.5*(x/s)**2)  # define kernel function
@@ -255,8 +264,7 @@ def gkernel(t,y, h = 3.0, step=None,start=None,stop=None, step_style = 'max'):
 
 
 def increments(x,step_style='median'):
-    ''' Establishes the increments of a numerical array:
-        start, stop, and representative step.
+    ''' Establishes the increments of a numerical array: start, stop, and representative step.
 
     Parameters
     ----------
@@ -278,7 +286,16 @@ def increments(x,step_style='median'):
         max(x)
     step : float
         The representative spacing between consecutive values, computed as above
+
+    See also
+    --------
+
+    pyleoclim.utils.tsutils.bin : Bin the values
+
+    pyleoclim.utils.tsutils.gkernel : Coarsen time resolution using a Gaussian kernel
+
     '''
+
     start = np.nanmin(x)
     stop = np.nanmax(x)
 
@@ -298,6 +315,8 @@ def increments(x,step_style='median'):
 def interp(x,y, interp_type='linear', step=None,start=None,stop=None, step_style= 'mean',**kwargs):
     """ Interpolate y onto a new x-axis
 
+    Largely a wrapper for [scipy.interpolate.interp1d](https://docs.scipy.org/doc/scipy/reference/generated/scipy.interpolate.interp1d.html)
+
     Parameters
     ----------
 
@@ -305,8 +324,11 @@ def interp(x,y, interp_type='linear', step=None,start=None,stop=None, step_style
        The x-axis
     y : array
        The y-axis
-    interp_type : {‘linear’, ‘nearest’, ‘zero’, ‘slinear’, ‘quadratic’, ‘cubic’, ‘previous’, ‘next’}
-        where ‘zero’, ‘slinear’, ‘quadratic’ and ‘cubic’ refer to a spline interpolation of zeroth, first, second or third order; ‘previous’ and ‘next’ simply return the previous or next value of the point) or as an integer specifying the order of the spline interpolator to use. Default is ‘linear’.
+    interp_type : str
+        Options include: 'linear', 'nearest', 'zero', 'slinear', 'quadratic', 'cubic', 'previous', 'next'
+        where 'zero', 'slinear', 'quadratic' and 'cubic' refer to a spline interpolation of zeroth, first, second or third order; 
+        'previous' and 'next' simply return the previous or next value of the point) or as an integer specifying the order of the spline interpolator to use. 
+        Default is 'linear'.
     step : float
             The interpolation step. Default is mean spacing between consecutive points.
     start : float
@@ -314,8 +336,9 @@ def interp(x,y, interp_type='linear', step=None,start=None,stop=None, step_style
     stop : float
          where/when to stop the interpolation. Default is max.
     kwargs :  kwargs
-        Aguments specific to interpolate.interp1D. See scipy for details https://docs.scipy.org/doc/scipy/reference/generated/scipy.interpolate.interp1d.html
+        Aguments specific to interpolate.interp1D.
         If getting an error about extrapolation, you can use the arguments `bound_errors=False` and `fill_value="extrapolate"` to allow for extrapolation. 
+
     Returns
     -------
 
@@ -323,6 +346,16 @@ def interp(x,y, interp_type='linear', step=None,start=None,stop=None, step_style
         The interpolated x-axis
     yi : array
         The interpolated y values
+
+    See Also
+    --------
+
+    pyleoclim.utils.tsutils.increment : Establishes the increments of a numerical array
+
+    pyleoclim.utils.tsutils.bin : Bin the values
+
+    pyleoclim.utils.tsutils.gkernel : Coarsen time resolution using a Gaussian kernel
+
     """
 
         #Make sure x and y are numpy arrays
@@ -352,90 +385,8 @@ def interp(x,y, interp_type='linear', step=None,start=None,stop=None, step_style
     return xi, yi
 
 
-# def on_common_axis(x1, y1, x2, y2, method = 'interpolation', step=None, start=None, stop=None):
-#     """Places two timeseries on a common axis
-
-#     Note this function assumes that the time representation and units are the same (e.g., BP vs CE)
-
-#     Parameters
-#     ----------
-#     x1 : array
-#         x-axis values of the first timeseries
-#     y1 : array
-#         y-axis values of the first timeseries
-#     x2 : array
-#         x-axis values of the second timeseries
-#     y2 : array
-#         y-axis values of the second timeseries
-#     method : str
-#         Which method to use to get the timeseries on the same x axis.
-#         Valid entries: 'interpolation' (default, linear interpolation),
-#         'bin', 'None'. 'None' only cuts the timeseries to the common
-#         period but does not attempt to generate a common time axis
-#     step : float
-#         The interpolation step. Default is mean resolution
-#         of lowest resolution series
-#     start : float
-#         where/when to start. Default is the maximum of the minima of
-#         the two timeseries
-#     stop : float
-#         Where/when to stop. Default is the minimum of the maxima of
-#         the two timeseries
-
-#     Returns
-#     -------
-
-#     xi1, xi2 : array
-#         The interpolated x-axis
-#     interp_values1, interp_values2 : array
-#         the interpolated y-values
-#     """
-#     # make sure that x1, y1, x2, y2 are numpy arrays
-#     x1 = np.array(x1, dtype='float64')
-#     y1 = np.array(y1, dtype='float64')
-#     x2 = np.array(x2, dtype='float64')
-#     y2 = np.array(y2, dtype='float64')
-
-#     # Find the mean/max x-axis is not provided
-#     if start is None:
-#         start = np.nanmax([np.nanmin(x1), np.nanmin(x2)])
-#     if stop is None:
-#         stop = np.nanmin([np.nanmax(x1), np.nanmax(x2)])
-
-#     # Get the interp_step
-#     if step is None:
-#         step = np.nanmin([np.nanmean(np.diff(x1)), np.nanmean(np.diff(x2))])
-
-#     if method == 'interpolation':
-#     # perform the interpolation
-#         xi1, interp_values1 = interp(x1, y1, step=step, start=start,
-#                                 stop=stop)
-#         xi2, interp_values2 = interp(x2, y2, step=step, start=start,
-#                                 stop=stop)
-#     elif method == 'bin':
-#         xi1, interp_values1, _ , _ = bin(x1, y1, bin_size=step, start=start,
-#                                 stop=stop)
-#         xi2, interp_values2, _ , _ = bin(x2, y2, bin_size=step, start=start,
-#                                 stop=stop)
-#     elif method == None:
-#         min_idx1 = np.where(x1>=start)[0][0]
-#         min_idx2 = np.where(x2>=start)[0][0]
-#         max_idx1 = np.where(x1<=stop)[0][-1]
-#         max_idx2 = np.where(x2<=stop)[0][-1]
-
-#         xi1 = x1[min_idx1:max_idx1+1]
-#         xi2 = x2[min_idx2:max_idx2+1]
-#         interp_values1 = y1[min_idx1:max_idx1+1]
-#         interp_values2 = y2[min_idx2:max_idx2+1]
-
-#     else:
-#         raise KeyError('Not a valid interpolation method')
-
-#     return xi1, xi2, interp_values1, interp_values2
-
-
 def standardize(x, scale=1, axis=0, ddof=0, eps=1e-3):
-    """ Centers and normalizes a given time series. Constant or nearly constant time series not rescaled.
+    """Centers and normalizes a time series. Constant or nearly constant time series not rescaled.
 
     Parameters
     ----------
@@ -464,8 +415,9 @@ def standardize(x, scale=1, axis=0, ddof=0, eps=1e-3):
     References
     ----------
 
-    1. Tapio Schneider's MATLAB code: https://github.com/tapios/RegEM/blob/master/standardize.m
-    2. The zscore function in SciPy: https://github.com/scipy/scipy/blob/master/scipy/stats/stats.py
+    Tapio Schneider's MATLAB code: https://github.com/tapios/RegEM/blob/master/standardize.m
+
+    The zscore function in SciPy: https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.zscore.html
 
     See also
     --------
@@ -504,9 +456,9 @@ def center(y, axis=0):
     ----------
 
     y : array
-        vector of (real) numbers as a time series, NaNs allowed
+        Vector of (real) numbers as a time series, NaNs allowed
     axis : int or None
-        axis along which to operate, if None, compute over the whole array
+        Axis along which to operate, if None, compute over the whole array
         
     Returns
     -------
@@ -518,6 +470,7 @@ def center(y, axis=0):
 
     References
     ----------
+
     Tapio Schneider's MATLAB code: https://github.com/tapios/RegEM/blob/master/center.m
 
     """
@@ -550,17 +503,17 @@ def ts2segments(ys, ts, factor=10):
     ts : array
         The time points
     factor : float
-        the factor that adjusts the threshold for gap detection
+        The factor that adjusts the threshold for gap detection
 
     Returns
     -------
 
     seg_ys : list
-        a list of several segments with potentially different lengths
+        A list of several segments with potentially different lengths
     seg_ts : list
-        a list of the time axis of the several segments
+        A list of the time axis of the several segments
     n_segs : int
-        the number of segments
+        The number of segments
     '''
 
     ys, ts = clean_ts(ys, ts)
@@ -625,80 +578,108 @@ def annualize(ys, ts):
     return ys_ann, year_int
 
 
-def gaussianize(X):
-    """ Transforms a (proxy) timeseries to a Gaussian distribution.
+# def gaussianize(X):
+#     """ Quantile maps a matrix to a Gaussian distribution
 
-    Originator: Michael Erb, Univ. of Southern California - April 2017
+#     Parameters
+#     ----------
 
+#     X : array
+#         Timeseries arrayed by column
+
+#     Returns
+#     -------
+
+#     Xn : array
+#         Gaussianized array
+        
+#     References
+#     ----------
+
+#     van Albada, S., and P. Robinson (2007), Transformation of arbitrary 
+#         distributions to the normal distribution with application to EEG 
+#         test-retest reliability, Journal of Neuroscience Methods, 161(2), 
+#         205 - 211, doi:10.1016/j.jneumeth.2006.11.004.    
+
+#     See also
+#     --------
+
+#     pyleoclim.utils.tsutils.gaussianize_1d : Quantile maps a 1D array to a Gaussian distribution
+
+#     """
+
+#     # Give every record at least one dimensions, or else the code will crash.
+#     X = np.atleast_1d(X)
+
+#     # Make a blank copy of the array, retaining the data type of the original data variable.
+#     Xn = copy.deepcopy(X)
+#     Xn[:] = np.NAN
+
+#     if len(X.shape) == 1:
+#         Xn = gaussianize_1d(X)
+#     else:
+#         for i in range(X.shape[1]):
+#             Xn[:, i] = gaussianize_1d(X[:, i])
+
+#     return Xn
+
+
+def gaussianize(ys):
+    """ Maps a 1D array to a Gaussian distribution using the inverse Rosenblatt transform
+    
+    The resulting array is mapped to a standard normal distribution, and therefore
+    has zero mean and unit standard deviation. Using `gaussianize()` obviates the 
+    need for `standardize()`. 
+    
     Parameters
     ----------
 
-    X : array
-        Values for the timeseries.
+    ys : 1D Array
+        e.g. a timeseries
 
     Returns
     -------
 
-    Xn : array
-        Gaussianized timseries
+    yg : 1D Array
+        Gaussianized values of ys.
 
-    """
-
-    # Give every record at least one dimensions, or else the code will crash.
-    X = np.atleast_1d(X)
-
-    # Make a blank copy of the array, retaining the data type of the original data variable.
-    Xn = copy.deepcopy(X)
-    Xn[:] = np.NAN
-
-    if len(X.shape) == 1:
-        Xn = gaussianize_single(X)
-    else:
-        for i in range(X.shape[1]):
-            Xn[:, i] = gaussianize_single(X[:, i])
-
-    return Xn
-
-
-def gaussianize_single(X_single):
-    """ Transforms a single (proxy) timeseries to Gaussian distribution.
-
-    Originator: Michael Erb, Univ. of Southern California - April 2017
-
-    Parameters
+    References
     ----------
+    
+    van Albada, S., and P. Robinson (2007), Transformation of arbitrary 
+        distributions to the normal distribution with application to EEG 
+        test-retest reliability, Journal of Neuroscience Methods, 161(2), 
+        205 - 211, doi:10.1016/j.jneumeth.2006.11.004.   
 
-    X_single : 1D Array
-        A single timeseries
+    See also
+    --------
 
-    Returns
-    -------
-
-    Xn_single : Gaussianized values for a single timeseries.
+    pyleoclim.utils.tsutils.standardize : Centers and normalizes a time series
 
     """
     # Count only elements with data.
 
-    n = X_single[~np.isnan(X_single)].shape[0]
+    n = ys[~np.isnan(ys)].shape[0]
 
     # Create a blank copy of the array.
-    Xn_single = copy.deepcopy(X_single)
-    Xn_single[:] = np.NAN
+    yg = copy.deepcopy(ys)
+    yg[:] = np.NAN
 
-    nz = np.logical_not(np.isnan(X_single))
-    index = np.argsort(X_single[nz])
+    nz = np.logical_not(np.isnan(ys))
+    index = np.argsort(ys[nz])
     rank = np.argsort(index)
     CDF = 1.*(rank+1)/(1.*n) - 1./(2*n)
-    Xn_single[nz] = np.sqrt(2)*special.erfinv(2*CDF - 1)
+    yg[nz] = np.sqrt(2)*special.erfinv(2*CDF - 1)
 
-    return Xn_single
+    return yg
 
 
 def detrend(y, x=None, method="emd", n=1, sg_kwargs=None):
     """Detrend a timeseries according to four methods
 
-    Detrending methods include, "linear", "constant", using a low-pass
-        Savitzky-Golay filter, and using Empirical Mode Decomposition (default).
+    Detrending methods include: "linear", "constant", using a low-pass Savitzky-Golay filter, and Empirical Mode Decomposition (default).
+    Linear and constant methods use [scipy.signal.detrend](https://docs.scipy.org/doc/scipy/reference/generated/scipy.signal.detrend.html),
+    EMD uses [pyhht.emd.EMD](https://pyhht.readthedocs.io/en/stable/apiref/pyhht.html)
 
     Parameters
     ----------
@@ -706,8 +687,8 @@ def detrend(y, x=None, method="emd", n=1, sg_kwargs=None):
     y : array
        The series to be detrended.
     x : array
-       The time axis for the timeseries. Necessary for use with
-       the Savitzky-Golay filters method since the series should be evenly spaced.
+       Abscissa for array y. Necessary for use with the Savitzky-Golay 
+       method, since the series should be evenly spaced.
     method : str
         The type of detrending:
         - "linear": the result of a linear least-squares fit to y is subtracted from y.
@@ -717,7 +698,7 @@ def detrend(y, x=None, method="emd", n=1, sg_kwargs=None):
     n : int
         Works only if `method == 'emd'`. The number of smoothest modes to remove.
     sg_kwargs : dict
-        The parameters for the Savitzky-Golay filters. see pyleoclim.utils.filter.savitzy_golay for details.
+        The parameters for the Savitzky-Golay filters.
 
     Returns
     -------
@@ -728,9 +709,9 @@ def detrend(y, x=None, method="emd", n=1, sg_kwargs=None):
     See also
     --------
 
-    pylecolim.utils.filter.savitzky_golay : Filtering using Savitzy-Golay
+    pyleoclim.utils.filter.savitzky_golay : Filtering using Savitzy-Golay
 
-    pylecolim.utils.tsutils.preprocess : pre-processes a times series using standardization and detrending.
+    pyleoclim.utils.tsutils.preprocess : pre-processes a times series using standardization and detrending.
 
     """
     y = np.array(y)
@@ -745,7 +726,7 @@ def detrend(y, x=None, method="emd", n=1, sg_kwargs=None):
     elif method == "savitzky-golay":
         # Check that the timeseries is uneven and interpolate if needed
         if x is None:
-            raise ValueError("A time axis is needed for use with the Savitzky-Golay filter method")
+            raise ValueError("An independent variable is needed for the Savitzky-Golay filter method")
         # Check whether the timeseries is unvenly-spaced and interpolate if needed
         if len(np.unique(np.diff(x)))>1:
             warnings.warn("Timeseries is not evenly-spaced, interpolating...")
@@ -777,16 +758,20 @@ def detrend(y, x=None, method="emd", n=1, sg_kwargs=None):
 def distance_neighbors(signal):
     '''Finds Distance of each point in the timeseries from its 4 nearest neighbors
 
-       Parameters
-       ----------
+    Wrapper around [sklearn.neighbors.NearestNeighbors](https://scikit-learn.org/stable/modules/generated/sklearn.neighbors.NearestNeighbors.html)
 
-       signal : array
-           The timeseries
+    Parameters
+    ----------
+
+    signal : array
+        The timeseries
+
+    Returns
+    -------
+
+    distances : array
+        Distance of each point from its nearest neighbors in decreasing order
 
-       Returns
-       -------
-       distances : array
-           Distance of each point from its nearest neighbors in decreasing order
     '''
     nn = NearestNeighbors(n_neighbors=4) # 4 nearest neighbors
     nbrs =nn.fit(signal.reshape(-1,1))
@@ -797,17 +782,18 @@ def distance_neighbors(signal):
 def find_knee(distances):
     '''Finds knee point automatically in a given array sorted in decreasing order
 
-       Parameters
-       ----------
+    Parameters
+    ----------
 
-       distances : array
-                  Distance of each point in the timeseries from it's nearest neighbors in decreasing order
+    distances : array
+        Distance of each point in the timeseries from it's nearest neighbors in decreasing order
 
-       Returns
-       -------
+    Returns
+    -------
+
+    knee : float
+        Knee point in the array
 
-      knee : float
-            knee point in the array
     '''
     nPoints = len(distances)
     allCoord = np.vstack((range(nPoints), distances)).T
@@ -829,46 +815,61 @@ def find_knee(distances):
 def detect_outliers(ts, ys,auto=True, plot_knee=True,plot_outliers=True,
                     plot_outliers_kwargs=None,plot_knee_kwargs=None,
                     figsize=[10,4],saveknee_settings=None,
-                    saveoutliers_settings=None,mute=False):
+                    saveoutliers_settings=None):
     ''' Function to detect outliers in the given timeseries
 
-       for more details, see: https://scikit-learn.org/stable/modules/generated/sklearn.cluster.DBSCAN.html
+    For more details, see: https://scikit-learn.org/stable/modules/generated/sklearn.cluster.DBSCAN.html
 
+    Parameters
+    ----------
 
-       Parameters
-       ----------
+    ts : array
+        Time axis of time series
+    ys : array
+        y values of time series
+    auto : boolean
+        True by default, if false the user manually selects the knee point
+    plot_knee : boolean
+        True by default, plots the knee
+    plot_outliers : boolean
+        True by default, plots the outliers using a scatter plot
+    plot_outliers_kwargs : dict
+        Keyword arguments for plot_scatter_xy for outliers plot
+    plot_knee_kwargs : dict
+        Keyword arguments for plot_xy for knee plot
+    figsize : tuple, list
+        Tuple or list of figure size
+    saveknee_settings : dict
+        the dictionary of arguments for plt.savefig() for knee plot; some notes below:
+        - "path" must be specified; it can be any existed or non-existed path,
+          with or without a suffix; if the suffix is not given in "path", it will follow "format"
+        - "format" can be one of {"pdf", "eps", "png", "ps"}
+    saveoutliers_settings : dict
+        the dictionary of arguments for plt.savefig() for outliers plot; some notes below:
+        - "path" must be specified; it can be any existed or non-existed path,
+          with or without a suffix; if the suffix is not given in "path", it will follow "format"
+        - "format" can be one of {"pdf", "eps", "png", "ps"}
 
-       ts : array
-            time axis of time series
-       ys : array
-            y values of time series
-       plot : boolean
-             true by default, plots the outliers using a scatter plot
-       auto : boolean
-             true by default, if false the user manually selects the knee point
-       mute : bool, optional
-            if True, the plot will not show;
-            recommend to turn on when more modifications are going to be made on ax
-            (going to be deprecated)
-       plot_kwargs : dict
-            keyword arguments for ax.plot()
+    Returns
+    -------
 
-       Returns
-       -------
+    outliers : array
+        a list of values consisting of outlier indices
 
-       outliers : array
-                   a list of values consisting of outlier indices
+    See also
+    --------
 
-       See also
-       --------
+    pyleoclim.utils.tsutils.distance_neighbors : Finds Distance of each point in the timeseries from its 4 nearest neighbors
 
-       pylecolim.utils.tsutils.distance_neighbors : Finds Distance of each point in the timeseries from its 4 nearest neighbors
+    pyleoclim.utils.tsutils.find_knee : Finds knee point automatically in a given array sorted in decreasing order
 
-       pylecolim.utils.tsustils.find_knee : Finds knee point automatically in a given array sorted in decreasing order
+    pyleoclim.utils.tsutils.remove_outliers : Removes outliers from a timeseries
 
-       pylecolim.utils.tsutils.remove_outliers : Removes outliers from a timeseries
+    pyleoclim.utils.plotting.plot_xy : Plot a timeseries
 
-       '''
+    pyleoclim.utils.plotting.plot_scatter_xy : Plot a scatter on top of a line plot.
+
+    '''
     #Take care of arguments for the knee plot
     saveknee_settings = {} if saveknee_settings is None else saveknee_settings.copy()
 
@@ -882,7 +883,7 @@ def detect_outliers(ts, ys,auto=True, plot_knee=True,plot_outliers=True,
         index = [i for i in range(len(distances))]
 
         if auto == True:
-            db = DBSCAN(eps=knee_point, min_samples=minpts)
+            db = DBSCAN(eps=knee_point, min_samples=int(minpts))
             clusters = db.fit(ys.reshape(-1, 1))
             cluster_labels = clusters.labels_
             outliers = np.where(cluster_labels == -1)
@@ -904,7 +905,7 @@ def detect_outliers(ts, ys,auto=True, plot_knee=True,plot_outliers=True,
                         arrowprops=dict(facecolor='black', shrink=0.05))
                 plot_xy(index, distances, xlabel='Indices', ylabel='Distances',plot_kwargs=plot_knee_kwargs,ax=ax1)
 
-            db = DBSCAN(eps=eps, min_samples=minpts)
+            db = DBSCAN(eps=eps, min_samples=int(minpts))
             clusters = db.fit(ys.reshape(-1, 1))
             cluster_labels = clusters.labels_
             outliers = np.where(cluster_labels == -1)
@@ -912,14 +913,11 @@ def detect_outliers(ts, ys,auto=True, plot_knee=True,plot_outliers=True,
         if 'fig1' in locals():
             if 'path' in saveknee_settings:
                 savefig(fig1, settings=saveknee_settings)
-            # else:
-            #     if not mute:
-            #         showfig(fig1)
 
         if plot_outliers==True:
             x2 = ts[outliers]
             y2 = ys[outliers]
-            plot_scatter_xy(ts,ys,x2,y2,figsize=figsize,xlabel='time',ylabel='value',savefig_settings=saveoutliers_settings,plot_kwargs=plot_outliers_kwargs, mute=mute)
+            plot_scatter_xy(ts,ys,x2,y2,figsize=figsize,xlabel='time',ylabel='value',savefig_settings=saveoutliers_settings,plot_kwargs=plot_outliers_kwargs)
 
         return outliers
 
@@ -941,9 +939,9 @@ def remove_outliers(ts,ys,outlier_points):
     ts : array
          x axis of timeseries
     ys : array
-        y axis of timeseries
-   outlier_points : array
-                   indices of outlier points
+        y axis of timeseries 
+    outlier_points : array
+        indices of outlier points
 
     Returns
     -------
@@ -955,7 +953,8 @@ def remove_outliers(ts,ys,outlier_points):
     See also
     --------
 
-    pylecolim.utils.tsutils.detect_outliers : Function to detect outliers in the given timeseries
+    pyleoclim.utils.tsutils.detect_outliers : Function to detect outliers in the given timeseries
+
     '''
 
     ys = np.delete(ys,outlier_points)
@@ -964,26 +963,25 @@ def remove_outliers(ts,ys,outlier_points):
     return ys,ts
 
 def eff_sample_size(y, detrend_flag=False):
-    '''
-    Effective Sample Size of timeseries y
+    '''Effective Sample Size of timeseries y
 
     Parameters
     ----------
     y : float 
        1d array 
        
-    detrend : boolean
+    detrend_flag : boolean
         if True (default), detrends y before estimation.         
 
     Returns
     -------
     neff : float
         The effective sample size
-        
-        
+    
     Reference
     ---------
-    Thiébaux HJ, Zwiers FW. 1984. The interpretation and estimation of
+
+    Thiébaux HJ and Zwiers FW, 1984: The interpretation and estimation of
     effective sample sizes. Journal of Climate and Applied Meteorology 23: 800–811.
 
     '''
@@ -1006,7 +1004,6 @@ def eff_sample_size(y, detrend_flag=False):
     
     return neff
 
-
 # alias
 std = standardize
 gauss = gaussianize
@@ -1025,11 +1022,12 @@ def preprocess(ys, ts, detrend=False, sg_kwargs=None,
         the Savitzky-Golay filters method since the series should be evenly spaced.
     detrend : string
         'none'/False/None - no detrending will be applied;
+        'emd' - the last mode is assumed to be the trend and removed from the series
         'linear' - a linear least-squares fit to `ys` is subtracted;
         'constant' - the mean of `ys` is subtracted
-        'savitzy-golay' - ys is filtered using the Savitzky-Golay filters and the resulting filtered series is subtracted from y.
+        'savitzy-golay' - ys is filtered using the Savitzky-Golay filter and the resulting filtered series is subtracted.
     sg_kwargs : dict
-        The parameters for the Savitzky-Golay filters. see pyleoclim.utils.filter.savitzy_golay for details.
+        The parameters for the Savitzky-Golay filter.
     gaussianize : bool
         If True, gaussianizes the timeseries
     standardize : bool
@@ -1044,7 +1042,13 @@ def preprocess(ys, ts, detrend=False, sg_kwargs=None,
     See also
     --------
 
-    pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay
+    pyleoclim.utils.tsutils.detrend : Detrend a timeseries according to four methods
+
+    pyleoclim.utils.filter.savitzy_golay : Filtering using Savitzy-Golay method
+
+    pyleoclim.utils.tsutils.standardize : Centers and normalizes a given time series
+
+    pyleoclim.utils.tsutils.gaussianize_1d : Quantile maps a matrix to a Gaussian distribution
 
     '''
 
